﻿using EonaCat.VolumeMixer.Helpers;
using EonaCat.VolumeMixer.Interfaces;
using System;
using System.Diagnostics;
using System.Threading.Tasks;

namespace EonaCat.VolumeMixer.Models
{
    // This file is part of the EonaCat project(s) which is released under the Apache License.
    // See the LICENSE file or go to https://EonaCat.com/License for full license details.
    public class AudioSession : IDisposable
    {
        private static Guid SharedAppSessionGuid = new Guid("c0a3a9d0-b4dc-58b7-b95e-dcc407dbd8db");
        private readonly IAudioSessionControlExtended _sessionControl;
        private IAudioVolume _audioVolume;
        private bool _isDisposed = false;
        private readonly object _syncLock = new object();

        public string DisplayName { get; private set; }
        public string IconPath { get; private set; }
        public uint ProcessId { get; private set; }
        public AudioSessionState State { get; private set; }

        internal AudioSession(IAudioSessionControlExtended sessionControl, IAudioSessionManager sessionManager)
        {
            _sessionControl = sessionControl;
            LoadSessionInfo();
            InitializeVolume(sessionManager);
        }

        private void InitializeVolume(IAudioSessionManager sessionManager)
        {
            lock (_syncLock)
            {
                try
                {
                    // Always use the shared static GUID
                    var result = sessionManager.GetAudioVolume(ref SharedAppSessionGuid, 0, out var ptr);
                    if (result == 0 && ptr != IntPtr.Zero)
                    {
                        _audioVolume = ComHelper.GetInterface<IAudioVolume>(ptr);
                    }
                }
                catch
                {
                    _audioVolume = null;
                }
            }
        }

        private void LoadSessionInfo()
        {
            lock (_syncLock)
            {
                try
                {
                    var result = _sessionControl.GetDisplayName(out var displayName);
                    DisplayName = result == 0 && !string.IsNullOrEmpty(displayName) ? displayName : string.Empty;

                    result = _sessionControl.GetIconPath(out var iconPath);
                    IconPath = result == 0 ? iconPath ?? "" : "";

                    result = _sessionControl.GetProcessId(out var processId);
                    ProcessId = result == 0 ? processId : 0;

                    result = _sessionControl.GetState(out var state);
                    State = result == 0 ? state : AudioSessionState.AudioSessionStateInactive;
                }
                catch
                {
                    DisplayName = string.Empty;
                    IconPath = string.Empty;
                    ProcessId = 0;
                    State = AudioSessionState.AudioSessionStateInactive;
                }
            }
        }

        public async Task<float> GetVolumeAsync()
        {
            return await Task.Run(() =>
            {
                lock (_syncLock)
                {
                    if (_isDisposed || _audioVolume == null)
                    {
                        return 0f;
                    }

                    try
                    {
                        var result = _audioVolume.GetMasterVolume(out var volume);
                        return result == 0 ? volume : 0f;
                    }
                    catch
                    {
                        return 0f;
                    }
                }
            });
        }

        public async Task<bool> SetVolumeAsync(float volume, int maxRetries = 2, int delayMs = 20)
        {
            if (volume < 0f || volume > 1f)
            {
                return false;
            }

            IAudioVolume audioVolCopy;
            lock (_syncLock)
            {
                if (_isDisposed || _audioVolume == null)
                {
                    return false;
                }

                audioVolCopy = _audioVolume;
            }

            var guid = Guid.Empty;

            for (int attempt = 0; attempt <= maxRetries; attempt++)
            {
                try
                {
                    var result = audioVolCopy.SetMasterVolume(volume, ref guid);
                    if (result == 0)
                    {
                        await Task.Delay(delayMs);

                        var currentVolume = await GetVolumeAsync();
                        if (Math.Abs(currentVolume - volume) < 0.01f)
                        {
                            return true;
                        }
                    }
                }
                catch
                {
                    // Retry
                }

                await Task.Delay(delayMs);
            }

            return false;
        }

        public async Task<bool> GetMuteAsync()
        {
            return await Task.Run(() =>
            {
                lock (_syncLock)
                {
                    if (_isDisposed || _audioVolume == null)
                    {
                        return false;
                    }

                    try
                    {
                        var result = _audioVolume.GetMute(out var mute);
                        return result == 0 ? mute : false;
                    }
                    catch
                    {
                        return false;
                    }
                }
            });
        }

        public async Task<bool> SetMuteAsync(bool mute)
        {
            IAudioVolume audioVolCopy;
            lock (_syncLock)
            {
                if (_isDisposed || _audioVolume == null)
                {
                    return false;
                }

                audioVolCopy = _audioVolume;
            }

            try
            {
                var guid = Guid.Empty;
                return await Task.Run(() => audioVolCopy.SetMute(mute, ref guid) == 0);
            }
            catch
            {
                return false;
            }
        }

        public async Task<string> GetProcessNameAsync()
        {
            return await Task.Run(() =>
            {
                lock (_syncLock)
                {
                    if (ProcessId == 0)
                    {
                        return "Unknown";
                    }

                    try
                    {
                        var process = Process.GetProcessById((int)ProcessId);
                        return process.ProcessName;
                    }
                    catch
                    {
                        return "Unknown";
                    }
                }
            });
        }

        public void Dispose()
        {
            lock (_syncLock)
            {
                if (_isDisposed)
                {
                    return;
                }

                ComHelper.ReleaseComObject(_audioVolume);
                ComHelper.ReleaseComObject(_sessionControl);
                _isDisposed = true;
            }
        }
    }
}
